package com.xy.job;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xy.collections.map.JHashMap;
import com.xy.collections.map.JMap;
import com.xy.entity.SysDictRedis;
import com.xy.mapper.DeviceChargingMapper;
import com.xy.sys.EnumMercCostMsgConfig;
import com.xy.utils.*;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * 设备管理费job
 */
@Slf4j
@Component
@AllArgsConstructor
public class DeviceChargingJob {

    private DeviceChargingMapper deviceChargingMapper;

    /**
     * 设备管理费过期消息
     *
     * @return
     */
    @XxlJob("deviceChargingTimeout")
    public ReturnT<String> deviceChargingTimeout() {
        String now = DataTime.getSring();
        Map<String, SysDictRedis> stringSysDictRedisMap = SysDictUtils.get(EnumMercCostMsgConfig.Code.CODE.getCode());
        String tel = stringSysDictRedisMap.get(EnumMercCostMsgConfig.CUSTOM_TEL.getCode()).toString();
        String mmiTime = DataTime.getStringAround(0, -1, 0, 0, 0, 0, now);
        //即将过期
        FunctionUtils.NoParamsNoResult timeout1 = () -> {
            int day = Integer.valueOf(stringSysDictRedisMap.get(EnumMercCostMsgConfig.DEVICE_COST_DAY.getCode()).toString());
            String end = DataTime.getStringAround(0, 0, day, 0, 0, 0, now);
            JMap<String, Object> map = new JHashMap<String, Object>()
                    .set("mmiTime", mmiTime)
                    .set("workType", 1)
                    .set("begin", now)
                    .set("end", end);
            deviceChargingTimeout1(map, 1, 100, tel);
        };
        //欠费
        FunctionUtils.NoParamsNoResult timeout2 = () -> {
            int day = Integer.valueOf(stringSysDictRedisMap.get(EnumMercCostMsgConfig.ARREARAGE_DAY.getCode()).toString());
            String time1 = DataTime.getStringAround(0, 0, day, 0, 0, 0, now);
            JMap<String, Object> map = new JHashMap<String, Object>()
                    .set("mmiTime", mmiTime)
                    .set("workType", 2)
                    .set("end", now);
            deviceChargingTimeout2(map, now, time1, 1, 100, tel);
        };
        timeout1.run();
        timeout2.run();
        return ReturnT.SUCCESS;
    }

    /**
     * 即将过期
     *
     * @param map     sql参数
     * @param current 当前页码
     * @param size    显示行数
     * @param tel     客服手机
     */
    private void deviceChargingTimeout1(JMap<String, Object> map, long current, long size, String tel) {
        //查询数据
        IPage<Long> longIPage = deviceChargingMapper.deviceChargingTimeout(Page.of(current, size), map);
        List<Long> mercIds = longIPage.getRecords();
        if (!Emptys.check(mercIds)) {
            return;
        }
        //发送消息
        JMap<String, Object> params = new JHashMap<String, Object>()
                .set("value", "您有设备管理费即将到期")
                .set("phone", tel);
        MsgUtils.sendMsg(15L, mercIds)
                .official(userInfo -> new MsgUtils.MsgInfo(userInfo.getMpOpenid(), params))
                .mini(userInfo -> new MsgUtils.MsgInfo(userInfo.getWechatOpenid(), params))
                .builder();
        //下一页
        current++;
        deviceChargingTimeout1(map, current, size, tel);
    }

    /**
     * 已欠费
     *
     * @param map      sql参数
     * @param thisTime 当前时间
     * @param time1    过期后可用时间
     * @param current  当前页码
     * @param size     显示行数
     * @param tel      客服手机
     */
    private void deviceChargingTimeout2(JMap<String, Object> map, String thisTime, String time1, long current, long size, String tel) {
        //查询数据
        IPage<Long> longIPage = deviceChargingMapper.deviceChargingTimeout(Page.of(current, size), map);
        List<Long> mercIds = longIPage.getRecords();
        if (!Emptys.check(mercIds)) {
            return;
        }
        //发送消息
        JMap<String, Object> params = new JHashMap<String, Object>()
                .set("time0", thisTime)
                .set("type", "设备管理费")
                .set("time1", time1)
                .set("phone", tel);
        MsgUtils.sendMsg(15L, mercIds)
                .official(userInfo -> new MsgUtils.MsgInfo(userInfo.getMpOpenid(), params))
                .mini(userInfo -> new MsgUtils.MsgInfo(userInfo.getWechatOpenid(), params))
                .builder();
        //下一页
        current++;
        deviceChargingTimeout2(map, thisTime, time1, current, size, tel);
    }

}
